﻿namespace SmartIdServer.API {
    /// <summary>
    ///     Команди API
    /// </summary>
    internal enum Commands {
        /// <summary>
        ///     Запит стану сервера
        /// </summary>
        ServerState = 0,

        /// <summary>
        ///     Запит сертифікату шифрування сервера
        /// </summary>
        GetEncryptCert = 1,

        /// <summary>
        ///     Запит на створення Заявки і секретних ключів (внутрішня для КНЕДП)
        /// </summary>
        GenerateKeys = 2,

        /// <summary>
        ///     Одержання результату запиту на створення Заявки і секретних ключів (внутрішня для КНЕДП)
        /// </summary>
        GenerateKeysResult = 3,

        /// <summary>
        ///     Запит на зміну пароля секретних ключів
        /// </summary>
        ChangeKeysPassword = 4,

        /// <summary>
        ///     Одержання результату запиту на зміну пароля секретних ключів
        /// </summary>
        ChangeKeysPasswordResult = 5,

        /// <summary>
        ///     Запит параметрів ключів
        /// </summary>
        GetKeysParameters = 6,

        /// <summary>
        ///     Запит на виконання криптографічних операцій
        /// </summary>
        DoOperations = 7,

        /// <summary>
        ///     Одержання результату запиту на виконання криптографічних операцій
        /// </summary>
        DoOperationsResult = 8,

        /// <summary>
        ///     Одержання операцій, що вимагають підтвердження користувачем
        /// </summary>
        GetOperationsForConfirmation = 9,

        /// <summary>
        ///     Підтвердження операцій користувачем
        /// </summary>
        ConfirmOperations = 10,

        /// <summary>
        ///     Підтвердження операції користувачем за QR-кодом (протокол IIT)
        /// </summary>
        ConfirmByQrCode = 11
    }

    /// <summary>
    ///     Код помилки
    /// </summary>
    internal enum ErrorCode {
        /// <summary>
        ///     OK
        /// </summary>
        Ok = 0,

        /// <summary>
        ///     Триває опрацювання запиту
        /// </summary>
        RequestProcessing = 1,

        /// <summary>
        ///     Очікується підтвердження користувача
        /// </summary>
        WaitingForUserConfirmation = 2,

        /// <summary>
        ///     Помилка криптографічних функцій
        /// </summary>
        CryptographyError = 3,

        /// <summary>
        ///     Запит не зареєстрований
        /// </summary>
        RequestNotFound = 4,

        /// <summary>
        ///     Помилка валідації даних
        /// </summary>
        DataValidationError = 5,

        /// <summary>
        ///     Некоректний параметр запиту
        /// </summary>
        InvalidQueryParameter = 6,

        /// <summary>
        ///     Відсутній доступ
        /// </summary>
        AccessNotGranted = 7,

        /// <summary>
        ///     Сертифікат не зареєстрований
        /// </summary>
        CertificateNotFound = 8,

        /// <summary>
        ///     Приватний ключ не зареєстрований
        /// </summary>
        PrivateKeyNotFound = 9,

        /// <summary>
        ///     Некоректний пароль приватного ключа
        /// </summary>
        InvalidPrivateKeyPassword = 10,

        /// <summary>
        ///     Перевищено дозволену кількість некоректних спроб введення пароля
        /// </summary>
        InvalidPasswordAttemptsExceeded = 11
    }

    internal static class ErrorHelper {
        public static string ErrorMessage( this ErrorCode code ) {
            return code switch {
                ErrorCode.Ok => "Ok",
                ErrorCode.RequestProcessing => "Триває опрацювання запиту",
                ErrorCode.WaitingForUserConfirmation => "Очікується підтвердження користувача",
                ErrorCode.CryptographyError => "Помилка криптографічних функцій",
                ErrorCode.RequestNotFound => "Запит не зареєстрований",
                ErrorCode.DataValidationError => "Помилка валідації даних",
                ErrorCode.InvalidQueryParameter => "Некоректний параметр запиту",
                ErrorCode.AccessNotGranted => "Відсутній доступ",
                ErrorCode.CertificateNotFound => "Сертифікат не зареєстрований",
                ErrorCode.PrivateKeyNotFound => "Приватний ключ не зареєстрований",
                ErrorCode.InvalidPrivateKeyPassword => "Некоректний пароль приватного ключа",
                ErrorCode.InvalidPasswordAttemptsExceeded =>
                    "Перевищено дозволену кількість некоректних спроб введення пароля",
                _ => null
            };
        }
    }
}
