﻿using System;
using System.IO;
using System.IO.Compression;
using System.Net;
using System.Net.Cache;
using System.Text;
using Newtonsoft.Json;

namespace SmartIdClient {
    internal class CWebClient : WebClient {
        private const int Timeout = 10 * 1000;

        private const string
            SmartIdSrvUrl = "http://sid.uakey.com.ua/smartid"; // "http://localhost/smartid";

        private const string OCSPSrvUrl = "http://uakey.com.ua/services/ocsp";

        public void PostCmd<TS, TR>( TS sendData, out TR rcvData ) {
            var str = JsonConvert.SerializeObject( sendData );

            var sdata = Encoding.UTF8.GetBytes( str );
            byte[] rdata = null;

            using( var resultStream = new MemoryStream() ) {
                using( var compressionStream = new GZipStream( resultStream, CompressionMode.Compress ) ) {
                    compressionStream.Write( sdata, 0, sdata.Length );
                }
                sdata = resultStream.ToArray();
            }

            Headers[HttpRequestHeader.ContentType] = "application/json";
            Headers[HttpRequestHeader.ContentEncoding] = "gzip";

            try {
                rdata = UploadData( SmartIdSrvUrl, sdata );
            }
            catch( WebException we ) when( we.Status == WebExceptionStatus.ProtocolError ) {
                // Помилка протокола
                throw;
            }
            catch {
                rdata = null;
            }
            if( rdata != null ) {
                str = Encoding.UTF8.GetString( rdata );
                rcvData = JsonConvert.DeserializeObject<TR>( str );
            }
            else {
                rcvData = default;
            }
        }

        public void SendData( byte[] sendData, out byte[] rcvData ) {
            Headers[HttpRequestHeader.ContentType] = "application/octet-stream";

            try {
                rcvData = UploadData( OCSPSrvUrl, "POST", sendData );
            }
            catch( WebException we ) when( we.Status == WebExceptionStatus.ProtocolError ) {
                // Помилка протокола
                throw;
            }
            catch {
                rcvData = null;
            }
        }

        protected override WebRequest GetWebRequest( Uri address ) {
            var request = base.GetWebRequest( address ) as HttpWebRequest;
            request.AutomaticDecompression = DecompressionMethods.Deflate | DecompressionMethods.GZip;
            request.CachePolicy = new HttpRequestCachePolicy( HttpRequestCacheLevel.BypassCache );
            request.Timeout = Timeout;
            return request;
        }
    }
}
