﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using CSPLibNET;
using Newtonsoft.Json;

namespace SmartIdClient {
    internal static class Common {
        public static byte[] CreateHash( byte[] data ) {
            return CSP.HashBlockData( data, null );
        }

        public static void GetCert( out byte[] cert, out byte[] privatKey, out byte[] privateKeyPassword ) {

            privateKeyPassword = null;//Encoding.GetEncoding( 1251 ).GetBytes( "123" );

            var res = CSP.CreateSelfSignedCertificate( null, CSP.DSTU4145_ALG, privateKeyPassword );
            privatKey = res.getContainer();
            cert = res.getRequest();
        }

        public static void EncryptDataSp( byte[] originatorPrivateKey,
            byte[] password, byte[] certOriginator,
            byte[] certRecipient,
            byte[] data,
            out byte[] envelope ) {
            envelope = CSP.EncryptDataSp( data, originatorPrivateKey, password, certOriginator, certRecipient, null, 1,
                0 );
        }

        public static void DecryptDataSp( byte[] recipientPrivateKey,
            byte[] password, byte[] certOriginator,
            byte[] certRecipient,
            byte[] envelope,
            out byte[] data ) {
            data = CSP.DecryptData( envelope, recipientPrivateKey, password, certOriginator, certRecipient, null );
        }

        public static void PostCmd_RequestDoOperationsRemote( string pass, string keyId, string b64Data,
            out ResponseDoOperations ret ) {
            ret = null;
            try {
                var requestDoOperations = new RequestDoOperations {
                    RemoteRequests = new Dictionary<string, OperRemoteRequest> {
                        {
                            keyId, new OperRemoteRequest {
                                Password = pass, RemoteRequestData = new Dictionary<int, OperRemoteRequestData> {
                                    {0, new OperRemoteRequestData {Data = b64Data}}
                                }
                            }
                        }
                    }
                };

                using var wc = new CWebClient();
                wc.PostCmd( requestDoOperations, out ret );
            }
            catch( Exception ex ) {
                Console.WriteLine( ex.Message );
            }
        }

        public static void PostCmd_RequestDoOperations( string pass, string keyId, string hash,
            out ResponseDoOperations ret ) {
            ret = null;
            try {
                var requestDoOperations = new RequestDoOperations {
                    SetSignatures = new Dictionary<string, OperSetSignature> {
                        {
                            keyId, new OperSetSignature {
                                Password = pass, SetSignaturesData =
                                    new Dictionary<int, OperSetSignatureData> {
                                        {0, new OperSetSignatureData {Hash = hash}}
                                    }
                            }
                        }
                    }
                };

                using var wc = new CWebClient();
                wc.PostCmd( requestDoOperations, out ret );
            }
            catch( Exception ex ) {
                Console.WriteLine( ex.Message );
            }
        }

        public static void PostCmd_RequestDoOperationsCrypt( string pass, string keyId, string hash,
            byte[] certRecipient, byte[] certOriginator, byte[] originatorPrivateKey, byte[] passwordOriginator,
            out ResponseDoOperations ret ) {
            ret = null;
            try {
                var requestDoOperations = new RequestDoOperations {
                    SetSignatures = new Dictionary<string, OperSetSignature> {
                        {
                            keyId, new OperSetSignature {
                                Password = pass, SetSignaturesData =
                                    new Dictionary<int, OperSetSignatureData> {
                                        {0, new OperSetSignatureData {Hash = hash}}
                                    }
                            }
                        }
                    }
                };

                var str = JsonConvert.SerializeObject( requestDoOperations );
                var sdata = Encoding.UTF8.GetBytes( str );

                EncryptDataSp( originatorPrivateKey, passwordOriginator, certOriginator,
                    certRecipient,
                    sdata,
                    out var envelope );

                if( envelope != null ) {
                    var req = new Request {
                        Cert = Convert.ToBase64String( certOriginator ),
                        Encrypted = Convert.ToBase64String( envelope )
                    };

                    using var wc = new CWebClient();
                    wc.PostCmd( req, out Response resp );

                    if( resp == null || resp.ErrorCode != ErrorCode.Ok ) {
                        return;
                    }
                    DecryptDataSp( originatorPrivateKey, passwordOriginator, certRecipient,
                        certOriginator, Convert.FromBase64String( resp.Encrypted ), out var data );

                    if( data != null ) {
                        str = Encoding.UTF8.GetString( data );
                        ret = JsonConvert.DeserializeObject<ResponseDoOperations>( str );
                    }
                }
                else {
                    throw new Exception( "CryptoError code: {res}" );
                }
            }
            catch( Exception ex ) {
                Console.WriteLine( ex.Message );
            }
        }

        public static void PostCmd_RequestDoOperationsResult( long requestId, string keyId,
            out ResponseDoOperationsResult ret ) {
            ret = null;
            try {
                var requestDoOperationsResult = new RequestDoOperationsResult {
                    RequestId = requestId,
                    SubjectKeyIds = new[] {keyId}
                };

                using var wc = new CWebClient();
                wc.PostCmd( requestDoOperationsResult, out ret );
                if( ret == null ) {
                    ret = new ResponseDoOperationsResult( ErrorCode.CrytographyError );
                }
            }
            catch( Exception ex ) {
                Console.WriteLine( ex.Message );
                ret = new ResponseDoOperationsResult( ErrorCode.CrytographyError );
            }
        }

        public static void PostCmd_RequestGetEncryptCert( out ResponseGetEncryptCert ret ) {
            ret = null;
            var req = new RequestGetEncryptCert();
            try {
                using var wc = new CWebClient();
                wc.PostCmd( req, out ret );
            }
            catch( Exception ex ) {
                Console.WriteLine( ex.Message );
            }
        }
    }
}
