﻿using System;
using System.Runtime.Serialization;

namespace SmartIdClient {
    [Serializable]
    internal class DataError : Exception {
        public DataError( ErrorCode code )
            : base( code.ErrorMessage() ) {
            Code = code;
        }

        public DataError( ErrorCode code, string message )
            : base( message ) {
            Code = code;
        }

        public DataError( ErrorCode code, string message, Exception innerException )
            : base( message, innerException ) {
            Code = code;
        }

        protected DataError( SerializationInfo serializationInfo, StreamingContext streamingContext )
            : base( serializationInfo, streamingContext ) {
        }

        public ErrorCode Code { get; }
    }

    [Serializable]
    internal class DataErrorInternal : Exception {
        public DataErrorInternal( string message )
            : base( message ) {
        }

        public DataErrorInternal( string message, Exception innerException )
            : base( message, innerException ) {
        }

        protected DataErrorInternal( SerializationInfo serializationInfo, StreamingContext streamingContext )
            : base( serializationInfo, streamingContext ) {
        }
    }

    [Serializable]
    internal class CryptographyErrorInternal : DataErrorInternal {
        public CryptographyErrorInternal( int result, string message )
            : base( message ) {
            CryptResult = result;
        }

        public CryptographyErrorInternal( int result, string message, Exception innerException )
            : base( message, innerException ) {
            CryptResult = result;
        }

        protected CryptographyErrorInternal( SerializationInfo serializationInfo, StreamingContext streamingContext )
            : base( serializationInfo, streamingContext ) {
        }

        public int CryptResult { get; }

        public int P11Result => GetP11Result( CryptResult );

        public static int GetP11Result( int code ) {
            return (code & (int)SmartIdClient.CryptResult.CSP_RES_ERROR_PKCS11) != 0
                ? code & ~(int)SmartIdClient.CryptResult.CSP_RES_ERROR_PKCS11
                : 0;
        }
    }
}
