﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using CSPLibNET;

namespace SmartIdClient {
    public static class RemoteContextTest {
        public static void Run() {
            SignTest();
            EncryptTest();
            DecryptTest();
            OCSPTest();
            MultiTest();
        }

        public static void SignTest() {
            var data = File.ReadAllBytes( "..\\..\\..\\UserData\\doc.pdf" );

            var cert = File.ReadAllBytes(
                "..\\..\\..\\UserData\\010000000000000000000000000000000789cd5d.crt"); //sign cert

            const string passForRemoteKey = "P12345678!";

            var certInfo = CSP.GetCertificateInfo( cert );

            var ctx = new RemoteContext( cert );

            var cadesInfo = new UACADESInfo();

            var requestIdHex = ctx.RequestCADES( data, cadesInfo );

            var request = ctx.GetRequest();

            var reqB64 = Convert.ToBase64String( request );

            Common.PostCmd_RequestDoOperationsRemote( passForRemoteKey, certInfo.subjectKeyIdentifier, reqB64,
                out var ret );

            var ret2 = new ResponseDoOperationsResult( ErrorCode.RequestProcessing );

            while( ret2.ErrorCode == ErrorCode.RequestProcessing ) {
                Common.PostCmd_RequestDoOperationsResult( ret.RequestId, certInfo.subjectKeyIdentifier, out ret2 );

                if( (int)ret2.ErrorCode > (int)ErrorCode.RequestProcessing ) {
                    return;
                }
                Thread.Sleep( 100 );
            }

            var remoteResponse = Convert.FromBase64String( ret2.RemoteRequest[certInfo.subjectKeyIdentifier]
                                                               .RemoteRequestData[0].RemoteRequestResponse );

            ctx.LoadResponse( remoteResponse );

            var remoteSign = ctx.GetCADES( requestIdHex ).getBytes();

            if( CSP.VerifyData( remoteSign, cert ) ) {
                var signData = CSP.GetSignedDataContent( remoteSign );
                if( signData != null ) {
                    File.WriteAllBytes( "..\\..\\..\\UserData\\signData.bin", signData );
                }
            }
        }

        public static void EncryptTest() {
            var data = File.ReadAllBytes( "..\\..\\..\\UserData\\doc.pdf" );

            var cert = File.ReadAllBytes(
                "..\\..\\..\\UserData\\010000000000000000000000000000000789cd5c.crt"); //crypt cert

            const string passForRemoteKey = "P12345678!";

            Common.GetCert( out var recipientCert, out var recipientKey, out var keypass );

            var certInfo = CSP.GetCertificateInfo( cert );

            var ctx = new RemoteContext( cert );

            var lst = new List<byte[]> {recipientCert};

            var requestIdHex = ctx.RequestEncrypted( data, lst.ToArray(), null, CSP.DEFAULT, 0 );

            var request = ctx.GetRequest();

            var reqB64 = Convert.ToBase64String( request );

            Common.PostCmd_RequestDoOperationsRemote( passForRemoteKey, certInfo.subjectKeyIdentifier, reqB64,
                out var ret );

            var ret2 = new ResponseDoOperationsResult( ErrorCode.RequestProcessing );

            while( ret2.ErrorCode == ErrorCode.RequestProcessing ) {
                Common.PostCmd_RequestDoOperationsResult( ret.RequestId, certInfo.subjectKeyIdentifier, out ret2 );

                if( (int)ret2.ErrorCode > (int)ErrorCode.RequestProcessing ) {
                    return;
                }
                Thread.Sleep( 100 );
            }

            var remoteResponse = Convert.FromBase64String( ret2.RemoteRequest[certInfo.subjectKeyIdentifier]
                                                               .RemoteRequestData[0].RemoteRequestResponse );

            ctx.LoadResponse( remoteResponse );

            var envelope = ctx.GetEncrypted( requestIdHex );

            Common.DecryptDataSp( recipientKey, keypass, cert, recipientCert, envelope, out var decryptData );
            if( decryptData != null ) {
                File.WriteAllBytes( "..\\..\\..\\UserData\\decryptData.pdf", decryptData );
            }
        }

        public static void DecryptTest() {
            var data = File.ReadAllBytes( "..\\..\\..\\UserData\\doc.pdf" );

            var certRecipient =
                File.ReadAllBytes("..\\..\\..\\UserData\\010000000000000000000000000000000789cd5c.crt"); //crypt cert

            const string passForRemoteKey = "P12345678!";

            Common.GetCert( out var certOriginator, out var keyOriginator, out var keypass );

            Common.EncryptDataSp( keyOriginator, keypass, certOriginator, certRecipient, data,
                out var envelope );

            var certInfo = CSP.GetCertificateInfo( certRecipient );

            var ctx = new RemoteContext( certRecipient );

            var requestIdHex = ctx.RequestDecrypted( envelope, certOriginator, null );

            var request = ctx.GetRequest();

            var reqB64 = Convert.ToBase64String( request );

            Common.PostCmd_RequestDoOperationsRemote( passForRemoteKey, certInfo.subjectKeyIdentifier, reqB64,
                out var ret );

            var ret2 = new ResponseDoOperationsResult( ErrorCode.RequestProcessing );

            while( ret2.ErrorCode == ErrorCode.RequestProcessing ) {
                Common.PostCmd_RequestDoOperationsResult( ret.RequestId, certInfo.subjectKeyIdentifier, out ret2 );

                if( (int)ret2.ErrorCode > (int)ErrorCode.RequestProcessing ) {
                    return;
                }
                Thread.Sleep( 100 );
            }

            var remoteResponse = Convert.FromBase64String( ret2.RemoteRequest[certInfo.subjectKeyIdentifier]
                                                               .RemoteRequestData[0].RemoteRequestResponse );

            ctx.LoadResponse( remoteResponse );

            var decryptData = ctx.GetDecrypted( requestIdHex );
            ;

            if( decryptData != null ) {
                File.WriteAllBytes( "..\\..\\..\\UserData\\decryptData2.pdf", decryptData );
            }
        }

        public static void OCSPTest() {
            var certRecipient =
                File.ReadAllBytes("..\\..\\..\\UserData\\010000000000000000000000000000000789cd5c.crt"); //crypt cert

            const string passForRemoteKey = "P12345678!";

            var ctx = new RemoteContext( certRecipient );

            var lst = new List<byte[]> {certRecipient};

            var requestIdHex = ctx.RequestOCSPRequest( lst.ToArray() );

            var request = ctx.GetRequest();

            var reqB64 = Convert.ToBase64String( request );

            var certInfo = CSP.GetCertificateInfo( certRecipient );

            Common.PostCmd_RequestDoOperationsRemote( passForRemoteKey, certInfo.subjectKeyIdentifier, reqB64,
                out var ret );

            var ret2 = new ResponseDoOperationsResult( ErrorCode.RequestProcessing );

            while( ret2.ErrorCode == ErrorCode.RequestProcessing ) {
                Common.PostCmd_RequestDoOperationsResult( ret.RequestId, certInfo.subjectKeyIdentifier, out ret2 );

                if( (int)ret2.ErrorCode > (int)ErrorCode.RequestProcessing ) {
                    return;
                }
                Thread.Sleep( 100 );
            }

            var remoteResponse = Convert.FromBase64String( ret2.RemoteRequest[certInfo.subjectKeyIdentifier]
                                                               .RemoteRequestData[0].RemoteRequestResponse );

            ctx.LoadResponse( remoteResponse );

            var ocsq = ctx.GetOCSPRequest( requestIdHex );

            if( ocsq != null ) {
                using var wc = new CWebClient();

                wc.SendData( ocsq, out var ocspResult );

                if( ocspResult != null ) {
                    File.WriteAllBytes( "..\\..\\..\\UserData\\ocspResult.bin", ocspResult );
                }
            }
        }

        public static void MultiTest() {
            var data = File.ReadAllBytes( "..\\..\\..\\UserData\\doc.pdf" );

            var cert = File.ReadAllBytes(
                "..\\..\\..\\UserData\\010000000000000000000000000000000789cd5c.crt"); //crypt cert

            var certRecipient =
                File.ReadAllBytes("..\\..\\..\\UserData\\010000000000000000000000000000000789cd5c.crt"); //crypt cert

            const string passForRemoteKey = "P12345678!";

            var certInfo = CSP.GetCertificateInfo( cert );

            var ctx = new RemoteContext( cert );

            var cadesInfo = new UACADESInfo();

            Common.GetCert( out var recipientCert, out var recipientKey, out var keypass );

            //sign
            var signRequestIdHex = ctx.RequestCADES( data, cadesInfo );

            //enc

            var lst = new List<byte[]> {recipientCert};

            var encryptRequestIdHex = ctx.RequestEncrypted( data, lst.ToArray(), null, CSP.DEFAULT, 0 );

            //dec
            Common.GetCert( out var certOriginator, out var keyOriginator, out var keypass2 );

            Common.EncryptDataSp( keyOriginator, keypass, certOriginator, certRecipient, data,
                out var envelope );

            var decryptRequestIdHex = ctx.RequestDecrypted( envelope, certOriginator, null );

            var request = ctx.GetRequest();

            var reqB64 = Convert.ToBase64String( request );

            Common.PostCmd_RequestDoOperationsRemote( passForRemoteKey, certInfo.subjectKeyIdentifier, reqB64,
                out var ret );

            var ret2 = new ResponseDoOperationsResult( ErrorCode.RequestProcessing );

            while( ret2.ErrorCode == ErrorCode.RequestProcessing ) {
                Common.PostCmd_RequestDoOperationsResult( ret.RequestId, certInfo.subjectKeyIdentifier, out ret2 );

                if( (int)ret2.ErrorCode > (int)ErrorCode.RequestProcessing ) {
                    return;
                }
                Thread.Sleep( 100 );
            }

            var remoteResponse = Convert.FromBase64String( ret2.RemoteRequest[certInfo.subjectKeyIdentifier]
                                                               .RemoteRequestData[0].RemoteRequestResponse );

            ctx.LoadResponse( remoteResponse );

            //sign
            var remoteSign = ctx.GetCADES( signRequestIdHex ).getBytes();

            if( CSP.VerifyData( remoteSign, cert ) ) {
                var signData = CSP.GetSignedDataContent( remoteSign );
                if( signData != null ) {
                    File.WriteAllBytes( "..\\..\\..\\UserData\\signData.bin", signData );
                }
            }

            //encrypt
            envelope = ctx.GetEncrypted( encryptRequestIdHex );

            Common.DecryptDataSp( recipientKey, keypass2, cert, recipientCert, envelope, out var decryptData );
            if( decryptData != null ) {
                File.WriteAllBytes( "..\\..\\..\\UserData\\decryptData.pdf", decryptData );
            }

            //decrypt
            decryptData = ctx.GetDecrypted( decryptRequestIdHex );
            ;

            if( decryptData != null ) {
                File.WriteAllBytes( "..\\..\\..\\UserData\\decryptData2.pdf", decryptData );
            }
        }
    }
}
